"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = _useState;

var _react = require("react");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

// Creates state management functions.
function _useState(_ref) {
  var initialState = _ref.initialState,
      onRender = _ref.onRender,
      items = _ref.items;

  // `VirtualScroller` state.
  //
  // The `_stateUpdate` variable shouldn't be used directly
  // because in some cases its value may not represent
  // the actual `state` of the `VirtualScroller`.
  //
  // * It will contain an incorrect initial value if `initialState` property is passed
  //   because it doesn't get initialized to `initialState`.
  //
  // * If `items` property gets changed, `state` reference variable gets updated immediately
  //   but the `_stateUpdate` variable here doesn't (until the component re-renders some other time).
  //
  // Instead, use the `state` reference below.
  //
  var _useState2 = (0, _react.useState)(),
      _useState3 = _slicedToArray(_useState2, 2),
      _stateUpdate = _useState3[0],
      _setStateUpdate = _useState3[1]; // This `state` reference is used for accessing the externally stored
  // virtual scroller state from inside a `VirtualScroller` instance.
  //
  // It's also the "source of truth" on the actual `VirtualScroller` state.
  //


  var state = (0, _react.useRef)(initialState); // Accumulates state updates until they have been applied.

  var targetState = (0, _react.useRef)(initialState); // Update the current state reference.
  //
  // Ignores the cases when `state` reference has already been updated
  // "immediately" bypassing a `_setStateUpdate()` call, because
  // in that case, `_stateUpdate` holds a stale value.
  //

  if (state.current !== targetState.current) {
    state.current = _stateUpdate;
  } // Call `onRender()` right after every state update.
  //
  // When `items` property changes, `useHandleItemsChange()` hook doesn't call
  // `_setStateUpdate()` because there's no need for a re-render.
  // But chaning `items` still does trigger a `VirtualScroller` state update,
  // so added `items` property in the list of this "effect"'s dependencies.
  //


  (0, _react.useLayoutEffect)(function () {
    onRender();
  }, [_stateUpdate, items]);
  return {
    getState: function getState() {
      return state.current;
    },
    // Updates existing state.
    //
    // State updates are incremental meaning that this code should mimick
    // the classic `React.Component`'s `this.setState()` behavior
    // when calling `this.setState()` doesn't replace `state` but rather merges
    // a set of the updated state properties with the rest of the old ones.
    //
    // The reason is that `useState()` updates are "asynchronous" (not immediate),
    // and simply merging over `...state` would merge over potentially stale
    // property values in cases when more than a single `updateState()` call is made
    // before the state actually updates, resulting in losing some of the state updates.
    //
    // For example, the first `updateState()` call updates shown item indexes,
    // and the second `updateState()` call updates `verticalSpacing`.
    // If it was simply `updateState({ ...state, ...stateUpdate })`
    // then the second state update could overwrite the first state update,
    // resulting in incorrect items being shown/hidden.
    //
    // Using `...state.current` instead of `...pendingState.current` here
    // would produce "stale" results.
    //
    updateState: function updateState(stateUpdate) {
      var newState = _objectSpread(_objectSpread({}, targetState.current), stateUpdate);

      targetState.current = newState; // If `items` property did change the component detects it at render time
      // and updates `VirtualScroller` items immediately by calling `.setItems()`.
      // But, since all of that happens at render time and not in an "effect",
      // if the state update was done as usual by calling `_setStateUpdate()`,
      // React would throw an error about updating state during render.
      // Hence, state update in that particular case should happen "directly",
      // without waiting for an "asynchronous" effect to trigger and call
      // an "asyncronous" `_setStateUpdate()` function.
      //
      // Updating state directly in that particular case works because there
      // already is a render ongoing, so there's no need to re-render the component
      // again after such render-time state update.
      //
      // When the initial `VirtualScroller` state is being set, it contains an `.items`
      // property too, but that initial setting is done using another function called
      // `setInitialState()`, so using `if (stateUpdate.items)` condition here for describing
      // just the case when `state` has been updated as a result of a `setItems()` call
      // seems to be fine.
      //

      if (stateUpdate.items) {
        // If a `stateUpdate` contains `items` then it means that there was a `setItems()` call.
        // No need to trigger a re-render — the component got re-rendered anyway.
        // Just update the `state` "in place".
        state.current = newState;
      } else {
        _setStateUpdate(newState);
      }
    }
  };
}
//# sourceMappingURL=useState.js.map