import { useMemo, useRef, useCallback } from 'react';
export default function useOnItemHeightChange(_ref) {
  var items = _ref.items,
      virtualScroller = _ref.virtualScroller;
  // Only compute the initial cache value once.
  var initialCacheValue = useMemo(function () {
    return new Array(items.length);
  }, []); // Handler functions cache.

  var cache = useRef(initialCacheValue); // Caches per-item `onItemHeightChange` functions' "references"
  // so that item components don't get re-rendered needlessly.

  var getOnItemHeightChange = useCallback(function (i) {
    if (!cache.current[i]) {
      cache.current[i] = function () {
        return virtualScroller.onItemHeightChange(i);
      };
    }

    return cache.current[i];
  }, [virtualScroller, cache]);
  return getOnItemHeightChange;
}
//# sourceMappingURL=useOnItemHeightChange.js.map